package ru.amse.javadependencies.zhukova.analyser;

import ru.amse.javadependencies.zhukova.data.dependencies.impl.ElementFeatureDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.ClassDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.TypeFeatureDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.DependencyGraph;
import ru.amse.javadependencies.zhukova.data.IClass;
import ru.amse.javadependencies.zhukova.data.IPackage;
import ru.amse.javadependencies.zhukova.data.impl.Package;
import ru.amse.javadependencies.zhukova.data.impl.Class;
import ru.amse.javadependencies.zhukova.util.Pair;
import ru.amse.javadependencies.zhukova.util.IPair;

import java.util.Set;
import java.util.HashSet;

/**
 * @author Anna R. Zhukova
 */
class StaticAnaliseMethods {
    private static final Set<String> ourPrimitiveTypesContractions = new HashSet<String>();
    static {
        // int
        ourPrimitiveTypesContractions.add("I");
        // double
        ourPrimitiveTypesContractions.add("D");
        // boolean
        ourPrimitiveTypesContractions.add("Z");
        // byte
        ourPrimitiveTypesContractions.add("B");
        // char
        ourPrimitiveTypesContractions.add("C");
        // short
        ourPrimitiveTypesContractions.add("S");
        // float
        ourPrimitiveTypesContractions.add("F");
        // long
        ourPrimitiveTypesContractions.add("J");
    }

    private static final Set<String> ourFilters = new HashSet<String>();
    static {
        /*ourFilters.add("java.lang");
        ourFilters.add("java.util");
        ourFilters.add("java.net");
        ourFilters.add("javax.swing");
        ourFilters.add("java.awt");
        ourFilters.add("java.nio");
        ourFilters.add("java.io");
        ourFilters.add("java.security");*/
        ourFilters.add("java.");
        ourFilters.add("javax.");
    }

    private static String getPackageName(String from) {
        int lastIndex = from.lastIndexOf('/');
        int firstIndex = 0;
        int lastArrayIndex = from.lastIndexOf('[');
        if (lastArrayIndex >= 0) {
            // [..[Lpackagename/../packagename/ClassName
            firstIndex = lastArrayIndex + 2;
        }
        int length = from.length();
        if (firstIndex >= length) firstIndex = length - 1;
        if (lastIndex < 0)  lastIndex = 0;                
        if (length == 0 || firstIndex >= lastIndex) {
            return "<default>";
        }
        return DependenciesAnalyser.replaceSlashesWithDots(from.substring(firstIndex, lastIndex));
    }

    private static String getClassName(String from) {
        while (from.startsWith("[")) {
            from = from.substring(from.indexOf('[') + 1);
        }
        from = from.substring(from.lastIndexOf('/') + 1);
        int index = from.indexOf(';');
        from = (index != -1) ? from.substring(0, from.length() - 1) : from;
        for (String primitiveContraction : ourPrimitiveTypesContractions) {
            if (primitiveContraction.equals(from)) {
                return null;
            }
        }
        return from;
    }

    private static boolean filter(String packageName) {
        for (String filter : ourFilters) {
            if (packageName.startsWith(filter)) {
                return false;
            }
        }
        return true;
    }

    static IClass initializeClass(DependencyGraph<IClass> classDependencyGraph, DependencyGraph<IPackage> packageDependencyGraph, String name) {
        String className = getClassName(name);
        String packageName = getPackageName(name);
        if (className == null || !filter(packageName)) {
            return null;
        }
        IPackage pckg = packageDependencyGraph.getClient(packageName);
        if (pckg == null) {
            pckg = new Package(packageName);
            packageDependencyGraph.put(packageName, pckg);
        }

        String fullName = packageName + "." + className;
        IClass clazz = classDependencyGraph.getClient(fullName);
        if (clazz == null) {
            clazz = new Class(className, pckg);
            classDependencyGraph.put(fullName, clazz);
        }
        pckg.addClass(clazz);
        return clazz;
    }

    static void addElementDependencies
            (DependencyGraph<IClass> classDependencyGraph, DependencyGraph<IPackage> packageDependencyGraph, IClass clazz, String desc,
             String name, String serverName, String arguments) {
        classDependencyGraph.addClient(clazz);
        IClass server = initializeClass(classDependencyGraph, packageDependencyGraph, desc);
        if (server != null) {
            ClassDependency clDependency = new ClassDependency(server);
            ElementFeatureDependency elFeatureDep = new ElementFeatureDependency(server);
            elFeatureDep.setClientFeatureName(name);
            elFeatureDep.setServerFeatureName(serverName + getArguments(arguments));
            clDependency.addSubdependency(elFeatureDep);
            clazz.addClassDependency(clDependency);
            clazz.getPackage().addClassDependency(clDependency);
        }
    }


    static void addTypeDependencies(DependencyGraph<IClass> classDependencyGraph, DependencyGraph<IPackage> packageDependencyGraph, IClass clazz, String desc, String name, TypeFeatureDependency.Type type) {
        int begin = desc.indexOf('L');
        // if not a primitive type
        if (begin >= 0) {
            desc = desc.substring(begin + 1);
            classDependencyGraph.addClient(clazz);
            IClass server = initializeClass(classDependencyGraph, packageDependencyGraph, desc);
            if (server != null) {
                ClassDependency clDependency = new ClassDependency(server);
                TypeFeatureDependency typeFeatureDep = new TypeFeatureDependency(server, type);
                typeFeatureDep.setClientFeatureName(name);
                clDependency.addSubdependency(typeFeatureDep);
                clazz.addClassDependency(clDependency);
                clazz.getPackage().addClassDependency(clDependency);
            }
        }
    }

    static String getArguments(String arguments) {
        String args = "";
        if (arguments != null && arguments.indexOf(")") != -1) {
            args = String.format("(%s)",
                    getArgumentsTypes(arguments.substring(arguments.indexOf("(") + 1, arguments.indexOf(")"))));
        }
        return args;
    }

    private static String getArgumentsTypes(String s) {
        String s1 = s;
        StringBuilder result = new StringBuilder("");
        while (s1.length() > 0) {
            IPair<String, String> pr = getFirstType(s1);
            result.append(pr.getFirst()).append("; ");
            s1 = pr.getSecond();
        }
        return result.toString();
    }

    private static IPair<String, String> getFirstType(String s) {
        if (!"".equals(s)) {
            switch (s.charAt(0)) {
                case'Z':
                    return new Pair<String, String>("boolean", s.substring(1));
                case'C':
                    return new Pair<String, String>("char", s.substring(1));
                case'B':
                    return new Pair<String, String>("byte", s.substring(1));
                case'S':
                    return new Pair<String, String>("short", s.substring(1));
                case'I':
                    return new Pair<String, String>("int", s.substring(1));
                case'F':
                    return new Pair<String, String>("float", s.substring(1));
                case'J':
                    return new Pair<String, String>("long", s.substring(1));
                case'D':
                    return new Pair<String, String>("double", s.substring(1));
                case'[':
                    IPair<String, String> pr = getFirstType(s.substring(1));
                    return new Pair<String, String>(pr.getFirst() + "[]", pr.getSecond());
                case'L':
                    return getFirstNotPrimitiveType(s);
            }
        }
        return new Pair<String, String>("", "");
    }

    private static IPair<String, String> getFirstNotPrimitiveType(String s) {
        if (!"".equals(s) && s.indexOf("L") != -1) {
            int to = s.indexOf(";");
            String s1 = DependenciesAnalyser.replaceSlashesWithDots(s.substring(s.indexOf('L') + 1, to));
            int length = s.length();
            return new Pair<String, String>(s1, (to < length) ? s.substring(to + 1, length) : "");
        }
        return new Pair<String, String>("", "");
    }
}
