package ru.amse.javadependencies.zhukova.data.impl;

import ru.amse.javadependencies.zhukova.data.dependencies.IClassDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.IPackageDependency;
import ru.amse.javadependencies.zhukova.data.dependencies.impl.PackageDependency;
import ru.amse.javadependencies.zhukova.data.IJavaElement;

import java.util.Set;
import java.util.HashSet;
import java.util.HashMap;

/**
 * @author Anna R. Zhukova
 */
public abstract class JavaElement implements IJavaElement {
    private String myName = "";
    private final HashMap<String, IClassDependency> myClassDependencies = new HashMap<String, IClassDependency>();
    private final HashMap<String, PackageDependency> myPackageDependencies = new HashMap<String, PackageDependency>();
    public JavaElement() {}

    public JavaElement(String name) {
        myName = name;
    }

    public String getName() {
        return myName;
    }

    public Set<IClassDependency> getClassDependencies() {
        return new HashSet<IClassDependency>(myClassDependencies.values());
    }

    public Set<IPackageDependency> getPackageDependencies() {
        return new HashSet<IPackageDependency>(myPackageDependencies.values());
    }

    public boolean addClassDependency(IClassDependency classDependency) {
        boolean result = false;
        String className = classDependency.getServer().getName();
        String packageName = classDependency.getServer().getPackage().getName();
        if (!myClassDependencies.containsKey(className)) {
            myClassDependencies.put(className, classDependency);
            result = true;
        } else {
            myClassDependencies.get(className).getSubdependencies().addAll(classDependency.getSubdependencies());
        }
        if (myPackageDependencies.containsKey(packageName)) {
            PackageDependency dependency = myPackageDependencies.get(packageName);
            if (result) {
                dependency.addSubdependency(classDependency);
            } else {
                dependency.updateSubdependency(myClassDependencies.get(className));
            }
        } else {
            PackageDependency dependency = new PackageDependency(classDependency.getServer().getPackage());
            dependency.addSubdependency(classDependency);
            myPackageDependencies.put(packageName, dependency);
        }
        return result;
    }

    public boolean addPackageDependency(PackageDependency packageDependency) {
        String packageName = packageDependency.getServer().getName();
        if (!myPackageDependencies.containsKey(packageName)) {
            myPackageDependencies.put(packageName, packageDependency);
            return true;
        }
        return false;
    }

    public String toString() {
        StringBuilder result = new StringBuilder("name: ");
        result.append(getName());
        result.append("\n  Package Dependencies:\n");
        int i = 0;
        for (IPackageDependency pd : getPackageDependencies()) {
            result.append("   ");
            result.append(++i);
            result.append(". ");
            result.append(pd.toString());
        }
        return result.toString();
    }

    public boolean equals(Object obj) {
        if (obj == null || !obj.getClass().equals(this.getClass())) {
            return false;
        }
        return obj == this || ((IJavaElement) obj).getFullName().equals(getFullName());
    }

    public int hashCode() {
        return getFullName().hashCode();
    }
}
