package ru.amse.javadependencies.zhukova.model.impl;
import ru.amse.javadependencies.zhukova.ui.graphpane.IGraphPane;
import ru.amse.javadependencies.zhukova.model.IGraphModel;
import ru.amse.javadependencies.zhukova.model.IVertex;
import ru.amse.javadependencies.zhukova.model.IArc;

import javax.swing.*;
import java.util.*;
import java.awt.*;

public class GraphModel extends Observable implements IGraphModel {
    private JTree myTree;
    private final Set<IVertex> myVertices = new HashSet<IVertex>();
    private final Set<SimpleVertex> mySimpleVertices = new HashSet<SimpleVertex>();
    private final Set<SuperVertex> mySuperVertices = new HashSet<SuperVertex>();
    private final Set<IArc> myArcs = new HashSet<IArc>();
    private Map<String, SuperVertex> myNameToSuperVertex = new HashMap<String, SuperVertex>();
    private Map<String, SimpleVertex> myNameToSimpleVertex = new HashMap<String, SimpleVertex>();
    private final IGraphPane myGraphPane;

    public GraphModel(IGraphPane gp) {
        myGraphPane = gp;
    }

    public void addVertex(IVertex v) {
        myVertices.add(v);
        v.setHidden(false);
        if (v instanceof SimpleVertex) {
            mySimpleVertices.add((SimpleVertex) v);
        } else if (v instanceof SuperVertex) {
            mySuperVertices.add((SuperVertex) v);
        }
        myGraphPane.checkPoint(v.getMaxPoint());
        setChanged();
        notifyObservers();
    }

    public LinkedList<IArc> removeVertex(IVertex v) {
        LinkedList<IArc> arcs = new LinkedList<IArc>();
        myVertices.remove(v);
        v.setHidden(true);
        if (v instanceof SimpleVertex) {
            mySimpleVertices.remove(v);
        } else if (v instanceof SuperVertex) {
            mySuperVertices.remove(v);
        }
        for (IArc arc : myArcs) {
            if (v.equals(arc.getFromVertex()) || v.equals(arc.getToVertex())) {
                arcs.add(arc);
            }
        }
        setChanged();
        notifyObservers(v);
        return arcs;
    }

    public void moveVertex(IVertex v, int dx, int dy) {
        Point location = v.getAbsoluteLocation();
        v.setLocation(new Point(location.x + dx, location.y + dy));
        myGraphPane.checkPoint(v.getMaxPoint());
        setChanged();
        notifyObservers();
    }

    public void addArc(IArc arc) {
        myArcs.add(arc);
        setChanged();
        notifyObservers();
    }

    public void removeArc(IArc arc) {
        myArcs.remove(arc);
        setChanged();
        notifyObservers();
    }

    public Set<IVertex> getVertices() {
        return Collections.unmodifiableSet(myVertices);
    }

    public Set<SimpleVertex> getSimpleVertices() {
        return Collections.unmodifiableSet(mySimpleVertices);
    }

    public Set<SuperVertex> getSuperVertices() {
        return Collections.unmodifiableSet(mySuperVertices);
    }

    public Set<IArc> getArcs() {
        return Collections.unmodifiableSet(myArcs);
    }

    public void clear() {
        myVertices.clear();
        mySimpleVertices.clear();
        mySuperVertices.clear();
        myArcs.clear();
    }

    public void setTree(JTree tree) {
        myTree = tree;
    }

    public JTree getTree() {
        return myTree;
    }

    public void setNameToSuperVertexMap(Map<String, SuperVertex> nameToVertex) {
        myNameToSuperVertex = nameToVertex;
    }

    public Map<String, SuperVertex> getNameToSuperVertexMap() {
        return Collections.unmodifiableMap(myNameToSuperVertex);
    }

    public void setNameToSimpleVertexMap(Map<String, SimpleVertex> nameToVertex) {
        myNameToSimpleVertex = nameToVertex;
    }

    public Map<String, SimpleVertex> getNameToSimpleVertexMap() {
        return Collections.unmodifiableMap(myNameToSimpleVertex);
    }
}