package ru.amse.javadependencies.zhukova.model.impl;

import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;
import ru.amse.javadependencies.zhukova.ui.graphpane.IGraphPane;

import java.awt.*;
import java.util.*;


public class SimpleVertex extends Vertex {
    private final Set<Arc> myArcs = new HashSet<Arc>();
    private final SuperVertex mySuperVertex;

    public SimpleVertex(Point location, String name, SuperVertex superVertex) {
        super(name);
        mySuperVertex = superVertex;
        setLocation(location);
    }

    public Set<Arc> getArcs() {
        return myArcs;
    }

    public void paint(Graphics g, GraphPane graphPane, boolean isSelected) {
        g.setFont(getFont());
        int width = getWidth();
        int height = getHeight();
        g.setColor(Color.BLACK);
        Point location = getAbsoluteLocation();
        g.drawRect(location.x, location.y,
                width, height);
        if (!isSelected) {
            g.setColor(graphPane.getBackground());
        } else {
            g.setColor(Color.LIGHT_GRAY);
        }
        g.fillRect(location.x + 1, location.y + 1,
                width - 1, height - 1);
        g.setColor(Color.BLACK);
        g.drawString(getName(), location.x + 5, location.y + getHeight() - 5);
    }

    public String getToolTipText() {
        return getSurname();
    }

    public boolean equals(Object obj) {
        if (!(obj instanceof SimpleVertex)) {
            return false;
        }
        SimpleVertex v = (SimpleVertex) obj;
        return this == v || getName().equals(v.getName()) && getSurname().equals(v.getSurname());
    }

    public int hashCode() {
        return getName().hashCode() + 7 * getSurname().hashCode();
    }

    public String getSurname() {
        return mySuperVertex.getName();
    }

    public void addArc(Arc arc) {
        myArcs.add(arc);
    }

    public void setLocation(Point p) {
        Point location = new Point(p);
        if (mySuperVertex != null) {
            Point pmin = mySuperVertex.getMinPoint();
            Point pmax = mySuperVertex.getMaxPoint();
            if (!mySuperVertex.isHidden()) {
                checkLocation(location, pmin, pmax);
            }
            location.x -= pmin.x;
            location.y -= pmin.y;
        }
        super.setLocation(location);
    }

    private void checkLocation(Point location, Point pmin, Point pmax) {
        location.x = Math.min(location.x, pmax.x - getWidth());
        location.x = Math.max(location.x, pmin.x);
        location.y = Math.min(location.y, pmax.y - getHeight());
        location.y = Math.max(location.y, pmin.y);
    }

    public Point getAbsoluteLocation() {
        Point location = new Point(getLocation());
        if (mySuperVertex != null) {
            Point pmin = mySuperVertex.getMinPoint();
            Point pmax = mySuperVertex.getMaxPoint();
            location.x += pmin.x;
            location.y += pmin.y;
            if (!mySuperVertex.isHidden()) {
                checkLocation(location, pmin, pmax);
            }
        }
        return location;
    }

    public SuperVertex getSuperVertex() {
        return mySuperVertex;
    }
}
