package ru.amse.javadependencies.zhukova.modelbuilding.tree.popupmenu;

import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;
import ru.amse.javadependencies.zhukova.ui.tree.CheckNode;
import ru.amse.javadependencies.zhukova.model.impl.Vertex;
import ru.amse.javadependencies.zhukova.model.IGraphModel;

import javax.swing.*;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;
import java.util.ArrayList;

/**
 * @author Anna R. Zhukova
 */
public class PopUpMenu extends JPopupMenu {
    private static JPanel ourTreePanel = new JPanel();
    private static GraphPane ourGraphPane;
    private static JTree ourTree;
    private static int ourRow;
    private static final PopUpMenu ourInstance = new PopUpMenu();

    private PopUpMenu() {
        super();
        add(Show.getInstance());
        add(ShowAll.getInstance());
        addSeparator();
        add(Hide.getInstance());
        add(HideAll.getInstance());
    }

    public static void setGraphPane(GraphPane gp) {
        ourGraphPane = gp;
    }

    public static GraphPane getGraphPane() {
        return ourGraphPane;
    }

    private static void enable(boolean enableShow, boolean enableShowAll, boolean enableHide, boolean enableHideAll) {
        Show.getInstance().setEnabled(enableShow);
        Hide.getInstance().setEnabled(enableHide);
        ShowAll.getInstance().setEnabled(enableShowAll);
        HideAll.getInstance().setEnabled(enableHideAll);
    }

    public static PopUpMenu getLeafShowInstance() {
        enable(true, false, false, false);
        return ourInstance;
    }

    public static PopUpMenu getLeafHideInstance() {
        enable(false, false, true, false);
        return ourInstance;
    }

    public static PopUpMenu getNotLeafShowInstance() {
        enable(true, true, false, true);
        return ourInstance;
    }

    public static PopUpMenu getNotLeafHideInstance() {
        enable(false, true, true, true);
        return ourInstance;
    }

    public static void setTree(JTree tree, JPanel treePanel) {
        ourTreePanel = treePanel;
        ourTree = tree;
    }

    public static void setRow(int row) {
        ourRow = row;
    }

    public static JTree getTree() {
        return ourTree;
    }

    public static int getRow() {
        return ourRow;
    }

    public static void updateTreePanel() {
        ourTreePanel.repaint();
    }

    public static void visitRow(boolean select, int selectionMode, boolean hide) {
        TreePath treePath = ourTree.getPathForRow(ourRow);
        if (treePath != null) {
            CheckNode node = (CheckNode) treePath.getLastPathComponent();
            ArrayList<CheckNode> nodes = new ArrayList<CheckNode>();
            node.setSelected(select, selectionMode, nodes);
            IGraphModel gm = ourGraphPane.getGraphModel();
            for (CheckNode checkNode : nodes) {
                boolean leaf = checkNode.isLeaf();
                String name = checkNode.toString();
                Vertex v;
                if (leaf) {
                    if (checkNode.getParent() != null) {
                        name = checkNode.getParent().toString() + "." + name;
                    }
                    v = gm.getNameToSimpleVertexMap().get(name);
                } else {
                    v = gm.getNameToSuperVertexMap().get(name);
                }
                if (v != null) {
                    if (hide) {
                        gm.removeVertex(v);
                    } else {
                        gm.addVertex(v);
                    }
                }
            }
            ((DefaultTreeModel) ourTree.getModel()).nodeChanged(node);
            // if node is root
            if (ourRow == 0) {
                ourTree.revalidate();
                ourTree.repaint();
            }
            getGraphPane().repaint();
            updateTreePanel();
        }
    }
}
