package ru.amse.javadependencies.zhukova.ui;

import ru.amse.javadependencies.zhukova.model.impl.GraphModel;
import ru.amse.javadependencies.zhukova.model.impl.Arc;
import ru.amse.javadependencies.zhukova.model.IArcFilter;
import ru.amse.javadependencies.zhukova.model.IGraphModel;
import ru.amse.javadependencies.zhukova.ui.undo.UndoManager;
import ru.amse.javadependencies.zhukova.ui.undo.IUndoManager;
import ru.amse.javadependencies.zhukova.ui.menuactions.Exit;
import ru.amse.javadependencies.zhukova.ui.menuactions.Undo;
import ru.amse.javadependencies.zhukova.ui.menuactions.Redo;
import ru.amse.javadependencies.zhukova.ui.menuactions.Open;
import ru.amse.javadependencies.zhukova.modelbuilding.tree.popupmenu.PopUpMenu;
import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;
import ru.amse.javadependencies.zhukova.ui.graphpane.tools.SelectingTool;
import ru.amse.javadependencies.zhukova.ui.tree.CheckNode;
import ru.amse.javadependencies.zhukova.modelbuilding.DependencyTreeBuilder;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;

/**
 * @author Anna R. Zhukova
 */
public class Main {
    private boolean myChanged;
    private JFrame myFrame;

    //panels
    //graph panel
    private final GraphPane myGraphPane = new GraphPane();
    private final JScrollPane myGraphScrollPane = new JScrollPane(myGraphPane, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    //tree panels
    // packages tree panel
    private final JPanel myTreePanel = new JPanel();
    private final JScrollPane myTreeScrollPane = new JScrollPane(myTreePanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // dependencies tree panel
    private final JPanel myDependencyTreePanel = new JPanel();
    private final JScrollPane myDependencyTreeScrollPane = new JScrollPane(myDependencyTreePanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    // trees
    private JTree myTree;
    private final JTree myDependencyTree;

    // split panes
    private final JSplitPane myTreeSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, myTreeScrollPane, myDependencyTreeScrollPane);
    private final JSplitPane mySplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, myGraphScrollPane, myTreeSplitPane);

    public Main() {
        FileChoosers.setMain(this);
        myDependencyTree = DependencyTreeBuilder.buildDependencyTree(this);
    }

    public static void main(String[] args) {
        new Main().getFrame();
    }

    public JFrame getFrame() {
        if (myFrame == null) {
            myFrame = new JFrame("Java Dependencies");
            Exit.setFrame(myFrame);
            Open.setMain(this);
            myFrame.setSize(1200, 750);
            myFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
            myFrame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    if (myChanged) {
                        Exit.confirmExit();
                    } else {
                        System.exit(0);
                    }
                }
            });
            myFrame.setJMenuBar(Menu.getMenuBar());
            myFrame.getContentPane().setLayout(new BorderLayout());
            myFrame.getContentPane().add(ToolBar.getToolBar(), BorderLayout.NORTH);
            myFrame.getContentPane().add(mySplitPane);
            initDependencyTreePanel();
            initSizes();
            initTreePanel();
            myFrame.setVisible(true);
        }
        return myFrame;
    }

    private void initDependencyTreePanel() {
        myDependencyTreePanel.add(myDependencyTree);
    }

    private void initTreePanel() {
        myTreePanel.setLayout(new BorderLayout());
        final JCheckBox onlyPackagesSelectionButton = new JCheckBox("Select only packages", false);
        onlyPackagesSelectionButton.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                SelectingTool.setOnlySuperVerticesSelection(onlyPackagesSelectionButton.isSelected());
                myGraphPane.deselectVertices();
            }
        });
        myTreePanel.add(onlyPackagesSelectionButton, BorderLayout.SOUTH);
    }

    private void initSizes() {
        myGraphPane.setSize(myFrame.getSize());
        myGraphPane.setPreferredSize(myFrame.getPreferredSize());
        mySplitPane.setSize(myFrame.getSize());
        mySplitPane.setPreferredSize(myFrame.getPreferredSize());
        mySplitPane.setDividerLocation(1.0);
        int width = myTreeSplitPane.getSize().width;
        Dimension d = new Dimension(width, myFrame.getHeight());
        myTreeSplitPane.setSize(d);
        myTreeSplitPane.setPreferredSize(d);
        myTreeSplitPane.setDividerLocation(0.7);
    }

    public void setGraphModel(GraphModel graphModel) {
        IUndoManager undoManager = new UndoManager();
        Undo.setUndoManager(undoManager);
        Redo.setUndoManager(undoManager);
        myGraphPane.setGraphModel(graphModel, undoManager);
        PopUpMenu.setGraphPane(myGraphPane);
        mySplitPane.setDividerLocation(0.7);
    }

    public IGraphModel getGraphModel() {
        return myGraphPane.getGraphModel();
    }

    public GraphPane getGraphPane() {
        return myGraphPane;
    }

    public JPanel getDependencyTreePanel() {
        return myDependencyTreePanel;
    }

    public void setTree(JTree tree) {
        if (myTree != null) {
            myTreePanel.remove(myTree);
        }
        myTree = tree;
        myTreePanel.add(myTree, BorderLayout.CENTER);
        PopUpMenu.setTree(myTree, myTreePanel);
        myTreePanel.repaint();
    }

    public void updateDependencyTree() {
        IArcFilter filter = Arc.getFilter();
        updateChildren(filter, (CheckNode) myDependencyTree.getModel().getRoot(), "");
        myDependencyTree.revalidate();
        myDependencyTree.repaint();
        myDependencyTreePanel.repaint();
    }

    private void updateChildren(IArcFilter filter, CheckNode parent, String namePrefix) {
        for (Enumeration nodes = parent.children(); nodes.hasMoreElements();) {
            CheckNode currentNode = (CheckNode) nodes.nextElement();
            String name = ((namePrefix.length() > 0) ? namePrefix + ": " : "") + currentNode.toString();
            if (filter.isABarrier(name)) {
                currentNode.setSelected(false);
            } else {
                currentNode.setSelected(true);
            }
            updateChildren(filter, currentNode, name);
        }
    }

    public void setChanged(boolean b) {
        myChanged = b;
    }
}
