package ru.amse.javadependencies.zhukova.ui.graphpane;

import ru.amse.javadependencies.zhukova.model.*;
import ru.amse.javadependencies.zhukova.model.impl.GraphModel;
import ru.amse.javadependencies.zhukova.model.impl.SuperVertex;
import ru.amse.javadependencies.zhukova.model.impl.SimpleVertex;
import ru.amse.javadependencies.zhukova.ui.undo.UndoManager;
import ru.amse.javadependencies.zhukova.ui.undo.IUndoManager;
import ru.amse.javadependencies.zhukova.ui.graphpane.tools.ToolTipMouseMotionListener;
import ru.amse.javadependencies.zhukova.ui.graphpane.tools.Tool;
import ru.amse.javadependencies.zhukova.ui.graphpane.tools.SelectingTool;
import ru.amse.javadependencies.zhukova.ui.graphpane.tools.ITool;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.util.List;

public class GraphPane extends JPanel implements IGraphPane {
    private Set<IVertex> mySelectedVertices = new HashSet<IVertex>();
    private ITool myCurrentTool = SelectingTool.getInstance();
    private GraphModel myGraphModel = new GraphModel(this);

    private class MyObserver implements Observer {

        public void update(Observable o, Object arg) {
            if (arg != null) {
                IVertex v = (IVertex) arg;
                for (IVertex selectedVertex : mySelectedVertices) {
                    if (selectedVertex.equals(v)) {
                        mySelectedVertices.remove(v);
                        break;
                    }
                }
            }
            repaint();
        }
    }

    public GraphPane() {
        super();
        setLayout(new BorderLayout());
        setSize(750, 750);
        setPreferredSize(new Dimension(750, 750));
        setBackground(new Color(250, 250, 170));
        setFocusable(true);
        addMouseMotionListener(new ToolTipMouseMotionListener(this));
        Tool.setGraphModel(this, new UndoManager());
        addMouseListener(myCurrentTool);
        addMouseMotionListener(myCurrentTool);
    }


    public void checkPoint(Point p) {
        int width = getWidth();
        int height = getHeight();
        if (p.x >= width) {
            width = p.x + 10;
        }
        if (p.y >= height) {
            height = p.y + 10;
        }
        Dimension d = new Dimension(width, height);
        setSize(d);
        setPreferredSize(d);
    }

    public void deselectVertices() {
        mySelectedVertices = new HashSet<IVertex>();
    }

    public void selectVertex(IVertex v) {
        mySelectedVertices.add(v);
    }

    public void setGraphModel(GraphModel gr, IUndoManager undoManager) {
        deselectVertices();
        myGraphModel = gr;
        myGraphModel.addObserver(new MyObserver());
        Tool.setGraphModel(this, undoManager);
        revalidate();
        repaint();
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        ((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        if (myGraphModel != null) {
            for (IArc arc : myGraphModel.getArcs()) {
                arc.paintIfNeeded(g);
            }
            List<SimpleVertex> vertices = new LinkedList<SimpleVertex>(myGraphModel.getSimpleVertices());
            for (ListIterator<SimpleVertex> it = vertices.listIterator(vertices.size()); it.hasPrevious(); ) {
                IVertex v = it.previous();
                v.paintIfNeeded(g, this, mySelectedVertices.contains(v));
            }
            List<SuperVertex> svertices = new LinkedList<SuperVertex>(myGraphModel.getSuperVertices());
            for (ListIterator<SuperVertex> it = svertices.listIterator(svertices.size()); it.hasPrevious(); ) {
                IVertex v = it.previous();
                v.paintIfNeeded(g, this, mySelectedVertices.contains(v));
            }
            myCurrentTool.paint(g);
        }
    }

    public IGraphModel getGraphModel() {
        return myGraphModel;
    }

    public Set<IVertex> getSelectedVertices() {
        return Collections.unmodifiableSet(mySelectedVertices);
    }

    public void setTool(Tool tool) {
        removeMouseListener(myCurrentTool);
        removeMouseMotionListener(myCurrentTool);
        myCurrentTool = tool;
        addMouseListener(myCurrentTool);
        addMouseMotionListener(myCurrentTool);
        repaint();
    }
}
