package ru.amse.javadependencies.zhukova.ui.graphpane.tools;

import ru.amse.javadependencies.zhukova.ui.graphpane.GraphPane;
import ru.amse.javadependencies.zhukova.model.impl.SuperVertex;

import java.awt.*;
import java.awt.event.MouseEvent;
import static java.lang.Math.round;
import static java.lang.Math.cos;
import static java.lang.Math.PI;
import static java.lang.Math.sin;

/**
 * @author Anna R. Zhukova
 */
public class ResizingTool extends Tool {
    private static boolean ourMoveLeftBorder;
    private static boolean ourMoveTopBorder;
    private static boolean ourMoveRightBorder;
    private static boolean ourMoveBottomBorder;
    private static Point ourFrom;
    private static SuperVertex ourSuperVertex;

    private static final ResizingTool ourInstance = new ResizingTool();

    private ResizingTool() {
        super();
    }

    public static ResizingTool getInstance(Point from, SuperVertex v,
                                                    boolean moveLeft, boolean moveRight,
                                                    boolean moveTop, boolean moveBottom) {
        ourMoveLeftBorder = moveLeft;
        ourMoveRightBorder = moveRight;
        ourMoveTopBorder = moveTop;
        ourMoveBottomBorder = moveBottom;
        ourFrom = from;
        ourSuperVertex = v;
        return ourInstance;
    }

    public void mouseDragged(MouseEvent e) {
        Point mouseLocation = e.getPoint();
        int dx = mouseLocation.x - ourFrom.x;
        int dy = mouseLocation.y - ourFrom.y;
        GraphPane graphPane = Tool.getGraphPane();
        if (ourMoveLeftBorder) {
            ourSuperVertex.moveLeftBorder(dx);
        }
        if (ourMoveRightBorder) {
            ourSuperVertex.moveRightBorder(dx);
        }
        if (ourMoveBottomBorder) {
            ourSuperVertex.moveBottomBorder(dy);
        }
        if (ourMoveTopBorder) {
            ourSuperVertex.moveTopBorder(dy);
        }
        ourFrom = mouseLocation;
        graphPane.repaint();
    }

    public void mouseReleased(MouseEvent e) {
        Tool.getGraphPane().setTool(SelectingTool.getInstance());
    }

    public void paint(Graphics g) {
        double direction = getArrowDirection();
        int x1 = (int) (ourFrom.x + 10 * Math.cos(direction));
        int y1 = (int) (ourFrom.y + 10 * Math.sin(direction));
        paintArrow(direction, x1, y1, Math.PI / 8, 5, g);
        direction += Math.PI;
        int x2 = (int) (ourFrom.x + 10 * Math.cos(direction));
        int y2 = (int) (ourFrom.y + 10 * Math.sin(direction));
        paintArrow(direction, x2, y2, Math.PI / 8, 5, g);
        g.drawLine(x1, y1, x2, y2);
    }

    private double getArrowDirection() {
        if (ourMoveLeftBorder) {
            if (ourMoveTopBorder) {
                return Math.PI / 4;
            }
            if (ourMoveBottomBorder) {
                return Math.PI * 3 / 4;
            }
            return 0;
        }
        if (ourMoveRightBorder) {
            if (ourMoveTopBorder) {
                return Math.PI * 3 / 4;
            }
            if (ourMoveBottomBorder) {
                return Math.PI / 4;
            }
            return 0;
        }
        return Math.PI / 2;
    }

    private static void paintArrow(double direction, int x0, int y0, double beta, int arrowLength, Graphics g) {
        g.setColor(Color.RED);
        int xr = x0 + (int) round((arrowLength * cos(PI + direction - beta)));
        int yr = y0 + (int) round((arrowLength * sin(PI + direction - beta)));
        int xl = x0 + (int) round((arrowLength * cos(PI + direction + beta)));
        int yl = y0 + (int) round((arrowLength * sin(PI + direction + beta)));

        g.drawLine(x0, y0, xr, yr);
        g.drawLine(x0, y0, xl, yl);
    }
}
