package ru.amse.javadependencies.zhukova.ui.undo;

import ru.amse.javadependencies.zhukova.ui.commands.ICommand;
import ru.amse.javadependencies.zhukova.ui.commands.Composite;

import java.util.*;

public class UndoManager implements IUndoManager {
    private boolean myNeedJoin = false;
    private final List<ICommand> myUndoList = new LinkedList<ICommand>();
    private final List<ICommand> myRedoList = new LinkedList<ICommand>();

    public void execute(ICommand command, boolean needJoin, boolean doIt) {
        myRedoList.clear();
        if (myUndoList.isEmpty() || !myNeedJoin) {
            myUndoList.add(command);
        } else {
            Composite composite = new Composite();
            composite.addCommand(myUndoList.remove(myUndoList.size() - 1));
            composite.addCommand(command);
            myUndoList.add(composite);
        }
        myNeedJoin = needJoin;
        if (doIt) {
            command.doIt();
        }
    }

    public void undo() {
        if (!myUndoList.isEmpty()) {
            ICommand c = myUndoList.remove(myUndoList.size() - 1);
            myRedoList.add(c);
            c.undo();
        }
    }

    public void update() {
        myNeedJoin = false;
    }

    public void redo() {
        if (!myRedoList.isEmpty()) {
            ICommand c = myRedoList.remove(myRedoList.size() - 1);
            myUndoList.add(c);
            c.doIt();
        }
    }
}
